@@ -2,24 +2,24 @@ class MyError(Exception):
     pass
 
 class TestClass(object):
-    def div(s, arg1, arg2):
+    def div(self, arg1, arg2):
         return arg1/arg2
-    def error(s):
+    def error(self):
         raise ValueError('a valueerror! Great!')
-    def error2(s):
+    def error2(self):
         return ValueError('a valueerror! Great!')
-    def othererr(s):
+    def othererr(self):
         raise MyError('my error!')
-    def othererr2(s):
+    def othererr2(self):
         return MyError('my error!')
-    def complexerror(s):
+    def complexerror(self):
         x=Foo()
         x.crash()
 
 
 class Foo(object):
-    def crash(s):
-        s.crash2('going down...')
-    def crash2(s, arg):
+    def crash(self):
+        self.crash2('going down...')
+    def crash2(self, arg):
         # this statement will crash on purpose:
         x=arg//2
