@@ -1,3 +1,4 @@
+from __future__ import print_function
 import time
 import Pyro4
 
@@ -31,7 +32,6 @@ daemon=Pyro4.core.Daemon()
 obj=Server()
 uri=daemon.register(obj)
 ns=Pyro4.naming.locateNS()
-ns.remove("example.oneway")
 ns.register("example.oneway", uri)
 print("Server is ready.")
 daemon.requestLoop()
