@@ -1,3 +1,4 @@
+from __future__ import print_function
 import time
 import Pyro4
 
@@ -19,7 +20,6 @@ ns=Pyro4.naming.locateNS()
 daemon=Pyro4.core.Daemon()
 obj=RemoteObject()
 uri=daemon.register(obj)
-ns.remove("example.proxysharing")
 ns.register("example.proxysharing", uri)
 print("Server is ready.")
 daemon.requestLoop()
