@@ -17,7 +17,8 @@ class Configuration(object):
                "POLLTIMEOUT", "THREADING2", "ONEWAY_THREADED",
                "DETAILED_TRACEBACK", "SOCK_REUSE",
                "THREADPOOL_MINTHREADS", "THREADPOOL_MAXTHREADS",
-               "THREADPOOL_IDLETIMEOUT", "HMAC_KEY", "AUTOPROXY")
+               "THREADPOOL_IDLETIMEOUT", "HMAC_KEY", "AUTOPROXY",
+               "BROADCAST_ADDRS", "NATHOST", "NATPORT")
 
     def __init__(self):
         self.reset()
@@ -32,6 +33,8 @@ class Configuration(object):
         self.NS_PORT = 9090      # tcp
         self.NS_BCPORT = 9091    # udp
         self.NS_BCHOST = None
+        self.NATHOST = None
+        self.NATPORT = 0
         self.COMPRESSION = False
         self.SERVERTYPE = "thread"
         self.DOTTEDNAMES = False   # server-side
@@ -46,6 +49,7 @@ class Configuration(object):
         self.THREADPOOL_IDLETIMEOUT = 5.0
         self.HMAC_KEY = None   # must be bytes type
         self.AUTOPROXY = True
+        self.BROADCAST_ADDRS = "<broadcast>, 0.0.0.0"   # comma separated list of broadcast addresses
 
         if useenvironment:
             # process enviroment variables
@@ -79,6 +83,19 @@ class Configuration(object):
             result[item]=getattr(self,item)
         return result
 
+    def parseAddressesString(self, addresses):
+        """
+        Parses the addresses string which contains one or more ip addresses separated by a comma.
+        Returns a sequence of these addresses. '' is replaced by the empty string.
+        """
+        result=[]
+        for addr in addresses.split(','):
+            addr=addr.strip()
+            if addr=="''":
+                addr=""
+            result.append(addr)
+        return result
+
     def dump(self):
         # easy config diagnostics
         from Pyro4.constants import VERSION
